/**HEADER********************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved                       
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: httpd_supp.h$
* $Version : 3.7.8.0$
* $Date    : Mar-9-2011$
*
* Comments:
*
*   HTTPD support functions header.
*
*END************************************************************************/

#ifndef HTTP_SUPP_H_
#define HTTP_SUPP_H_

#include "httpd_types.h"

/** Send string to socket.
 * \param s < socket
 * \param str < pointer to string
 * \return length of sent data
 */
_mqx_int httpd_sendstr(_mqx_int s, char *str);

/** Read data from tcp/ip stack.
 * \param session < pointer to session
 * \param dest < pointer to destination buffer
 * \param len < requested read length
 * \return real read length
 */
_mqx_int httpd_read(HTTPD_SESSION_STRUCT *session, char *dst, _mqx_int len);

/** Read line from tcp/ip stack.
 * \param session < pointer to session
 * \param dest < pointer to destination buffer
 * \param len < maximal read length
 * \return nonzero - read success
 */
_mqx_int httpd_readln(HTTPD_SESSION_STRUCT *session, char *dst, _mqx_int max_len);

/** send header
 * \param session < pointer to evaluated session
 * \param content_len < content len
 * \param has_entity < indicate that there will be HTTP sentitty after this header
 */
void httpd_sendhdr(HTTPD_SESSION_STRUCT *session, _mqx_int content_len, boolean has_entity);

/** send file
 * \param server < pointer to server configuration structure
 * \param session < pointer to evaluated session
 */
void httpd_sendfile(HTTPD_STRUCT *server, HTTPD_SESSION_STRUCT *session);

/** HTTP find function and call it.
 * \param table < pointer to table of functions
 * \param fn_name < cgi script name
 * \return
 *      < 0 - not found or function request flush stack
 *      other - read length from stack
 */
_mqx_int httpd_call_fn(HTTPD_FN_LINK_STRUCT *table, HTTPD_SESSION_STRUCT *session, char *fn_name);

/*
** Send error page. If whole page cannot be send, than send just header.
**
** IN:
**      char* title - Title of send web page.
**      char* text - Text shown on error web page.
**
** OUT:
**      none
**
** Return Value: 
**      none
*/

#define ERR_PAGE_FORMAT "<HTML><HEAD><TITLE>%s</TITLE></HEAD>\n<BODY><H1>%s</H1>\n</BODY></HTML>\n"

void httpd_send_err_page(HTTPD_SESSION_STRUCT *session, const char* title, const char* text);

#endif /* HTTP_SUPP_H_ */
